/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef MINRTSFFTUTIL_H
#define MINRTSFFTUTIL_H
#include <math.h>
#include "fourg.h"
#include "minrtsFilterUtil.h"
#define vmax(a,b) (a > b) ? a:b
#define vmin(a,b) (a < b) ? a:b
#define vabs(a) (a > 0) ? a: -a
#define COMPLEX   0
#define REAL      1
#define IMAGINARY 2
#define ABS       3

typedef struct /* struct for minrtsFft1d */
{
int nX;             /* number of points in x-space */
float convParm[2];  /* convolution parameters */
float gridData[3];  /* gridding parameters */
float *grid;        /* gridded data in k-space */
float *grid2;        /* gridded data in k-space */
float *gridWt;      /* weights at each gridded point */
float *kGrid;       /* values of k corresponding to grid Data */
float *filter;      /* values of filter in k-space */
float *gridCorr;    /* correction for gridding convolution */
float *work;        /* work array */
float *cWork;       /* complex work array */
} minrtsFft1d ;

void minrtsKillFft1d(minrtsFft1d *ff);
minrtsFft1d *minrtsMakeFft1d(float* gD, float *cP);
void minrtsFftSetRFilter (minrtsFft1d *ff, float *filter);
void minrtsFftSetCFilter (minrtsFft1d *ff, minrtsFilter *filter);
void minrtsFftMultCFilter (minrtsFft1d *ff, minrtsFilter *filter);
float *minrtsFftGetFilter(minrtsFft1d *ff);
float *minrtsFftGrid(minrtsFft1d *ff);
float *minrtsFftKGrid(minrtsFft1d *ff);
float *minrtsFftCoord(minrtsFft1d *ff);
float *minrtsFftGridCorr(minrtsFft1d *ff);
void   minrtsFftGridData (minrtsFft1d *ff, int nX, float *xPos, float *data, 
   float *data2, float *dataWeight); /* real input */
void   minrtsFftGridDataC (minrtsFft1d *ff, int nX, float *xPos, float *data, 
   float *dataWeight); /* complex input */
void minrtsFftGridFft (minrtsFft1d *ff, int nX, float *xPos, float *data, 
   float *dataWeight, float *fft);/*real*/
void minrtsFftGridFftC (minrtsFft1d *ff, int nX, float *xPos, float *data, 
   float *dataWeight, float *fft); /*complex*/
void minrtsFftGridFft2 (minrtsFft1d *ff, int nX, float *xPos, float *data, 
   float *data2, float *dataWeight, float **fft1, float **fft2); /* two real arrays */
void minrtsFftBackFft (minrtsFft1d *ff, float *inData, int oType, float *outData);
void minrtsFftApplyFilter (minrtsFft1d *ff, int filterType, float *data);
void minrtsFftFilterData (minrtsFft1d *ff, int nData, float *xPos,
   float *data , float *dataWeight, int filterType, int outputType, float *output);
float *minrtsMakeHermit (float *data, int nData);
void   minrtsDoubleRealFftF (int nData, float *data1, float *data2, 
   float **fft1, float**fft2);
void   minrtsDoubleRealFftB (int nData, float *fft1, float *fft2, 
float **real1, float **real2);
float minrtsDot (int n, float *v1, float *v2);
void  minrtsVCopy (int n, float *vout, float *vin);
void  minrtsVAdd (int n, float *v1, float *v2);
void  minrtsVMult (int n, float *v1, float *v2);
void  minrtsVSMult (int n, float *v1, float s);
void  minrtsVSMultAdd (int n, float *v1, float s1, float s2);
float minrtsQPk (int nData, float *x, float *y, float *ymax, int domin);
void minrtsExpC (int nData, float *phase, float *expc);
void minrtsRCMult (int nData, float *real, float *comp);
void minrtsCMult (int nData, float *c1, float *c2);
void minrtsCReverse (int nData, float *c1 );
void minrtsCConj (int nData, float *c1 );
void minrtsLinearFit(int nData, float *data, float *weight, float *fit);
void minrtsGaussSmooth(float sigma, int nData, float *data, float *output, minrtsFft1d **ff, int killFft); 
#endif /* MINRTSFFTUTIL_H */
